#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Thu Apr 23 11:28:26 2020

@author: Q. Wang
"""
import os
import re
import sys
import random
from scipy.spatial import KDTree
import numpy as np
def touch(fname, times=None):
    fhandle = open(fname, 'a')
    try:
        os.utime(fname, times)
    finally:
        fhandle.close()
def search_string_in_file(file_name, string_to_search):
    """Search for the given string in file and return lines containing that string,
    along with line numbers"""
    line_number = 0
    list_of_results = []
    # Open the file in read only mode
    with open(file_name, 'r') as read_obj:
        # Read all lines in the file one by one
        for line in read_obj:
            # For each line, check if line contains the string
            line_number += 1
            if string_to_search in line:
                # If yes, then add the line number & line as a tuple in the list
                list_of_results.append((line_number, line.rstrip()))
    # Return list of tuples containing line numbers and lines where string is found
    return list_of_results
FoamHeader = "/*--------------------------------*- C++ -*----------------------------------*\ \n\
  =========                 |\n\
  \\      /  F ield         | OpenFOAM: The Open Source CFD Toolbox\n\
   \\    /   O peration     | Website:  https://openfoam.org\n\
    \\  /    A nd           | Version:  sprint\n\
     \\/     M anipulation  |\n\
\*---------------------------------------------------------------------------*/\n\
FoamFile\n\
{\n\
    version     2.0;\n\
    format      ascii;\n\
    class       vectorField;\n\
    location    \"constant/positionsFile\";\n\
    object      positions;\n\
}\n\
// * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * //\n\
"
#It is noted that this script limits to the bubbly flow. That is, in the 
#setFieldsDict, only sphereToCell is availble
#Manually input or configure
#input your path files manually
#the 1. is hemera or local path, and the 2. is the docker path
#twohorizontalBubbles
#oneBubble
#twoverticalBubblesDiag
#twoverticalBubbles
casePath1 = '/home/wang77/bigdata/OpenFOAM/lagrangeVOF/tutorials/Luo/10mm/twoverticalBubbles_init_15mm'
casePath2 = '/root/OpenFOAM/root-sprint/tutorials/Luo/oneBubble'
if os.path.exists(casePath1):
    casePath = casePath1
elif os.path.exists(casePath2):
     casePath = casePath2
#inout the number of particles manually
#noP = input("Please input the number of particles\n")
#if paticles are not allowed to be found in the bubbles, then the actuall noP
# generated by the script may be smaller\
#than the value defined below
noP = 1000
#particles in bubbles pib, boolen--> True|False
pib = False
#geometry in 2D or 3D D3=True|False
#2D corresponds D3 = False
D3 = False
#if 2D, which direction is set as empty? x|y|z | None
empty = 'y'
#the defualt decimals of particle coordinates is 5
d = 5
#to avoid Particle initialised with a location outside of the mesh, 
#the bounding box is multiplied by a factor.
#when the particles diameter is larger, the factor f is smaller
fb = 0.8
#to avoid the particles is included into bubbles, a factor fr is multiplied. see below
fr = 1.1
#diameter of the particles
dp = 1e-4
#****************************************************************************#
filePath = os.path.join(casePath,'constant','positionsFile')
if not os.path.exists(filePath):
    touch(filePath)
fo= open(filePath,'w')
fo.write(FoamHeader)

#regular expression
regNum = '([-+]?[0-9]*\.?[0-9]+(?:[eE][-+]?[0-9]+)?)'

if os.path.exists(os.path.join(casePath,'constant','polyMesh','points')):
    fname = os.path.join(casePath,'constant','polyMesh','points')
    points = np.genfromtxt(fname, skip_header = 18, max_rows=1)
    with open(fname,'rb') as f:
        clean_lines=(line.replace(b'(',b' ').replace(b')',b' ') for line in f)
        minxyz = np.genfromtxt(clean_lines, skip_header = 20, max_rows=1)
        maxxyz = np.genfromtxt(clean_lines, skip_header = int(points)-2, max_rows=1)
        xminOri,yminOri,zminOri = minxyz
        xmaxOri,ymaxOri,zmaxOri = maxxyz
elif os.path.exists(os.path.join(casePath,'log.blockMesh')):
    reg1='boundingBox: \('+regNum + ' ' + regNum + ' ' + regNum + '\) \('+regNum + ' ' + regNum + ' ' + regNum + '\)'
    prog1=re.compile(reg1)
    for i, line in enumerate(open(os.path.join(casePath,'log.blockMesh'))):
        for match1 in re.finditer(prog1, line):
            xminOri = np.float(match1.group(1))
            yminOri = np.float(match1.group(2))
            zminOri = np.float(match1.group(3))
            xmaxOri = np.float(match1.group(4))
            ymaxOri = np.float(match1.group(5))
            zmaxOri = np.float(match1.group(6))
else:
    print("cannot read mesh information\n Error!\nplease check if there exists polyMesh/points under constant dir or the log files named log.blockMesh under case dir")
    sys.exit(1)
xmean = (xminOri + xmaxOri) / 2
ymean = (yminOri + ymaxOri) / 2
zmean = (zminOri + zmaxOri) / 2
xmin = xmean - (xmean - xminOri)*(fb) 
xmax = xmean + (xmaxOri - xmean)*(fb) 
ymin = ymean - (ymean - yminOri)*(fb) 
ymax = ymean + (ymaxOri - ymean)*(fb) 
zmin = zmean - (zmean - zminOri)*(fb) 
zmax = zmean + (zmaxOri - zmean)*(fb) 
if not os.path.exists(os.path.join(casePath,'system','setFieldsDict')):
    print("please check if there exists setFieldsDict under system directories")
else:
    reg2='centre \('+regNum + ' ' + regNum + ' ' + regNum + '\);'
    prog2=re.compile(reg2)
    reg3='radius '+regNum+';'
    prog3=re.compile(reg3)
    ox=[]
    oy=[]
    oz=[]
    r=[]
    for i, line in enumerate(open(os.path.join(casePath,'system','setFieldsDict'))):
        for match2 in re.finditer(prog2, line):
            ox.append(np.float(match2.group(1)))
            oy.append(np.float(match2.group(2)))
            oz.append(np.float(match2.group(3)))
        for match3 in re.finditer(prog3, line):
            r.append(np.float(match3.group(1)))
#generate a random particles coordinates
#firstly generate a seed for the pseudo-random numbers in python
random.seed()
value = np.random.random_sample((noP,3))
valuex = value[:,0] * xmax
valuey = value[:,1] * ymax
valuez = value[:,2] * zmax
valueXY = np.vstack((valuex, valuey)).T
valueXZ = np.vstack((valuex, valuez)).T
valueYZ = np.vstack((valuey, valuez)).T
valueXYZ = np.vstack((valuex, valuey, valuez)).T
if D3 == True:
    tree = KDTree(valueXYZ)
    dictdok = tree.sparse_distance_matrix(tree, dp)
    keys = list(dictdok.keys())
    K0 = []
    for i, k in enumerate(keys):
        if i%2==0:
            K0.append(k)
    K = np.unique(K0)
    index = np.delete(np.asarray(range(0,noP)), K, 0)
    valueXYZ =  valueXYZ[index]
    valuex = np.round(valueXYZ[:,0],decimals = 5)
    valuey = np.round(valueXYZ[:,1],decimals = 5)
    valuez = np.round(valueXYZ[:,2],decimals = 5)
else:
    if empty == 'x':
        tree = KDTree(valueYZ)
        dictdok = tree.sparse_distance_matrix(tree, dp)
        keys = list(dictdok.keys())
        K0 = []
        for i, k in enumerate(keys):
            if i%2==0:
                K0.append(k)
        K = np.unique(K0)
        index = np.delete(np.asarray(range(0,noP)), K, 0)
        valueYZ =  valueYZ[index]
        valuey = np.round(valueYZ[:,0],decimals = 5)
        valuez = np.round(valueYZ[:,1],decimals = 5)
    elif empty == 'y':
        tree = KDTree(valueXZ)
        dictdok = tree.sparse_distance_matrix(tree, dp)
        keys = list(dictdok.keys())
        K0 = []
        for i, k in enumerate(keys):
            if i%2==0:
                K0.append(k)
        K = np.unique(K0)
        index = np.delete(np.asarray(range(0,noP)), K, 0)
        valueXZ =  valueXZ[index]
        valuex = np.round(valueXZ[:,0],decimals = 5)
        valuez = np.round(valueXZ[:,1],decimals = 5)
    elif empty == 'z':
        tree = KDTree(valueXY)
        dictdok = tree.sparse_distance_matrix(tree, dp)
        keys = list(dictdok.keys())
        K0 = []
        for i, k in enumerate(keys):
            if i%2==0:
                K0.append(k)
        K = np.unique(K0)
        index = np.delete(np.asarray(range(0,noP)), K, 0)
        valueXY =  valueXY[index]
        valuex = np.round(valueXY[:,0],decimals = 5)
        valuey = np.round(valueXY[:,1],decimals = 5)

#valuex=np.round(np.array(valuex),decimals = 5)
#valuey=np.round(np.array(valuey),decimals = 5)
#valuez=np.round(np.array(valuez),decimals = 5) 
ox=np.array(ox)    
oy=np.array(oy)    
oz=np.array(oz)     
r=np.array(r)            
#2D geometry:
if D3 == False:
    if empty == 'y':
        if pib == False:
            for i in range(len(ox)):
                index = np.where((valuex - ox[i])**2 + (valuez - oz[i])**2 > fr*r[i]**2 + dp**2/4.0 )
                valuex = valuex[index]
                valuez = valuez[index]
        fo.write(str(len(valuex)) + '\n')
        fo.write("(\n")
        #particles coordiantes, based on A Cartesian coordinate system, namely (x,y,z)
        #(x,y,z)
        for i in range(len(valuex)):
            fo.write("(")
            fo.write(str(valuex[i]) + ' ' +str(ymaxOri/2.0) + ' ' + str(valuez[i]))
            fo.write(")\n")
    elif empty == 'x':
        if pib == False:
            for i in range(len(ox)):
                index = np.where((valuey - oy[i])**2 + (valuez - oz[i])**2 > fr*r[i]**2 + dp**2/4.0 )
                valuey = valuey[index]
                valuez = valuez[index]
        fo.write(str(len(valuey)) + '\n')
        fo.write("(\n")
        #particles coordiantes, based on A Cartesian coordinate system, namely (x,y,z)
        #(x,y,z)
        for i in range(len(valuey)):
            fo.write("(")
            fo.write(str(xmaxOri/2) + ' ' +str(valuey[i]) + ' ' + str(valuez[i]))
            fo.write(")\n")
    elif empty == 'z':
        if pib == False:
            for i in range(len(ox)):
                index=np.where((valuex - ox[i])**2 + (valuey - oy[i])**2 > fr*r[i]**2 + dp**2/4.0 )
                valuex = valuex[index]
                valuey = valuey[index]
        fo.write(str(len(valuex)) + '\n')
        fo.write("(\n")
        #particles coordiantes, based on A Cartesian coordinate system, namely (x,y,z)
        #(x,y,z)
        for i in range(len(valuex)):
            fo.write("(")
            fo.write(str(valuex[i]) + ' ' +str(valuey[i]) + ' ' + str(zmaxOri/2))
            fo.write(")\n")
#3D geometry
elif D3 == True:
    if pib == False:
        for i in range(len(ox)):
            index=np.where((valuex - ox[i])**2 + (valuey - oy[i])**2 + (valuez - oz[i])**2 >fr* r[i]**2 + dp**2/4.0)
            valuex = valuex[index]
            valuey = valuey[index]
            valuez = valuez[index]
    fo.write(str(len(valuex)) + '\n')
    fo.write("(\n")
    #particles coordiantes, based on A Cartesian coordinate system, namely (x,y,z)
    #(x,y,z)
    for i in range(len(valuex)):
        fo.write("(")
        fo.write(str(valuex[i]) + ' ' +str(valuey[i]) + ' ' + str(valuez[i]))
        fo.write(")\n")    
fo.write(")\n")
fo.close()

